@@ -12,19 +12,30 @@ from abc import abstractmethod
 from collections import defaultdict
 from functools import cached_property
 
+import deprecation
+from earthkit.utils.array import array_namespace
+from earthkit.utils.array import array_to_numpy
+from earthkit.utils.array import convert_array
+from earthkit.utils.array import get_backend
+
 from earthkit.data.core import Base
 from earthkit.data.core.index import Index
 from earthkit.data.core.index import MaskIndex
 from earthkit.data.core.index import MultiIndex
 from earthkit.data.decorators import cached_method
 from earthkit.data.decorators import detect_out_filename
-from earthkit.data.utils.array import array_namespace
-from earthkit.data.utils.array import array_to_numpy
-from earthkit.data.utils.array import convert_array
-from earthkit.data.utils.array import get_backend
 from earthkit.data.utils.metadata.args import metadata_argument
 
 
+def _bits_per_value_to_metadata(**kwargs):
+    # TODO: remove this function when save() and write() are removed
+    metadata = {}
+    bits_per_value = kwargs.pop("bits_per_value", None)
+    if bits_per_value is not None:
+        metadata = {"bitsPerValue": bits_per_value}
+    return metadata, kwargs
+
+
 class FieldListIndices:
     def __init__(self, field_list):
         self.fs = field_list
@@ -252,6 +263,8 @@ class Field(Base):
         for k in keys:
             # TODO: convert dtype
             v = _keys[k](dtype=dtype)
+            if v is None:
+                raise ValueError(f"data: {k} not available")
             v = self._reshape(v, flatten)
             if index is not None:
                 v = v[index]
@@ -446,6 +459,8 @@ class Field(Base):
         dict of datatime.datetime
             Dict with items "base_time" and "valid_time".
 
+        Examples
+        --------
         >>> import earthkit.data
         >>> ds = earthkit.data.from_source("file", "tests/data/t_time_series.grib")
         >>> ds[4].datetime()
@@ -666,6 +681,7 @@ class Field(Base):
         """
         return self._metadata.dump(namespace=namespace, **kwargs)
 
+    @deprecation.deprecated(deprecated_in="0.13.0", details="Use to_target() instead")
     def save(self, filename, append=False, **kwargs):
         r"""Write the field into a file.
 
@@ -678,15 +694,45 @@ class Field(Base):
             the target file be overwritten if already exists. Default is False
         **kwargs: dict, optional
             Other keyword arguments passed to :obj:`write`.
-
-        See Also
-        --------
-        :obj:`write`
-
         """
-        flag = "wb" if not append else "ab"
-        with open(filename, flag) as f:
-            self.write(f, **kwargs)
+        metadata = kwargs.pop("metadata", None)
+        if metadata is None:
+            metadata, kwargs = _bits_per_value_to_metadata(**kwargs)
+        self.to_target("file", filename, append=append, metadata=metadata, **kwargs)
+        # the original implementation
+        # flag = "wb" if not append else "ab"
+        # with open(filename, flag) as f:
+        #     self.write(f, **kwargs)
+
+    @deprecation.deprecated(deprecated_in="0.13.0", details="Use to_target() instead")
+    def write(self, f, **kwargs):
+        metadata = kwargs.pop("metadata", None)
+        if metadata is None:
+            metadata, kwargs = _bits_per_value_to_metadata(**kwargs)
+        self.to_target("file", f, metadata=metadata, **kwargs)
+
+    def to_target(self, target, *args, **kwargs):
+        r"""Write the field into a target object.
+
+        Parameters
+        ----------
+        target: object
+            The target object to write the field into.
+        *args: tuple
+            Positional arguments used to specify the target object.
+        **kwargs: dict, optional
+            Other keyword arguments used to write the field into the target object.
+        """
+        from earthkit.data.targets import to_target
+
+        to_target(target, *args, data=self, **kwargs)
+
+    def default_encoder(self):
+        return self._metadata.data_format()
+
+    def _encode(self, encoder, **kwargs):
+        """Double dispatch to the encoder"""
+        return encoder._encode_field(self, **kwargs)
 
     def __getitem__(self, key):
         """Return the value of the metadata ``key``."""
@@ -1045,8 +1091,9 @@ class FieldList(Index):
         if n > 0:
             it = iter(self)
             first = next(it)
-            is_property = isinstance(getattr(first.__class__, accessor), property)
+            is_property = not callable(getattr(first, accessor, None))
             vals = _vals(first)
+            first = None
             ns = array_namespace(vals)
             shape = (n, *vals.shape)
             r = ns.empty(shape, dtype=vals.dtype)
@@ -1336,6 +1383,8 @@ class FieldList(Index):
         ls
         tail
 
+        Notes
+        -----
         The following calls are equivalent:
 
         .. code-block:: python
@@ -1372,6 +1421,8 @@ class FieldList(Index):
         head
         ls
 
+        Notes
+        -----
         The following calls are equivalent:
 
         .. code-block:: python
@@ -1412,6 +1463,8 @@ class FieldList(Index):
         dict of datatime.datetime
             Dict with items "base_time" and "valid_time".
 
+        Examples
+        --------
         >>> import earthkit.data
         >>> ds = earthkit.data.from_source("file", "tests/data/t_time_series.grib")
         >>> ds.datetime()
@@ -1572,6 +1625,7 @@ class FieldList(Index):
                 return all(f._metadata.geography._unique_grid_id() == grid for f in self)
         return False
 
+    @deprecation.deprecated(deprecated_in="0.13.0", removed_in=None, details="Use to_target() instead")
     @detect_out_filename
     def save(self, filename, append=False, **kwargs):
         r"""Write all the fields into a file.
@@ -1593,10 +1647,15 @@ class FieldList(Index):
         :meth:`SimpleFieldList.save() <data.indexing.fieldlist.SimpleFieldList.save>`
 
         """
-        flag = "wb" if not append else "ab"
-        with open(filename, flag) as f:
-            self.write(f, **kwargs)
+        metadata, kwargs = _bits_per_value_to_metadata(**kwargs)
+        self.to_target("file", filename, append=append, metadata=metadata, **kwargs)
 
+        # original code
+        # flag = "wb" if not append else "ab"
+        # with open(filename, flag) as f:
+        #     self.write(f, **kwargs)
+
+    @deprecation.deprecated(deprecated_in="0.13.0", removed_in=None, details="Use to_target() instead")
     def write(self, f, **kwargs):
         r"""Write all the fields to a file object.
 
@@ -1612,8 +1671,20 @@ class FieldList(Index):
         read
 
         """
-        for s in self:
-            s.write(f, **kwargs)
+        metadata, kwargs = _bits_per_value_to_metadata(**kwargs)
+        self.to_target("file", f, metadata=metadata, **kwargs)
+
+        # original code
+        # for s in self:
+        #     s.write(f, **kwargs)
+
+    def default_encoder(self):
+        if len(self) > 0:
+            return self[0]._metadata.data_format()
+
+    def _encode(self, encoder, **kwargs):
+        """Double dispatch to the encoder"""
+        return encoder._encode_fieldlist(self, **kwargs)
 
     def to_tensor(self, *args, **kwargs):
         from earthkit.data.indexing.tensor import FieldListTensor
