@@ -7,6 +7,7 @@
 # nor does it submit to any jurisdiction.
 #
 
+import copy
 import logging
 
 from earthkit.data.utils import ensure_dict
@@ -16,8 +17,6 @@ from earthkit.data.utils.serialise import serialise_state
 
 LOG = logging.getLogger(__name__)
 
-_WARNED_ABOUT_EARTHKIT = False
-
 
 class ItemWrapperForCfGrib:
     def __init__(self, item, ignore_keys=[]):
@@ -75,7 +74,7 @@ class XarrayMixIn:
             chunks=None,  # Set to 'auto' for lazy loading
         )
 
-    def to_xarray(self, engine=None, xarray_open_dataset_kwargs=None, **kwargs):
+    def to_xarray(self, engine="earthkit", xarray_open_dataset_kwargs=None, **kwargs):
         """
         Convert the FieldList into an Xarray Dataset.
 
@@ -109,23 +108,74 @@ class XarrayMixIn:
             * variable_key: str, None
                 Metadata key to specify the dataset variables. It cannot be
                 defined as a dimension. Default is "param" (in earthkit-data this is the same as "shortName").
+                Only enabled when ``mono_variable`` is False or None.
             * drop_variables: str, or iterable of str, None
-                A variable or list of variables to drop from the dataset. Default is None.
+                A variable or list of variables to drop from the dataset. Default is None. Only used when
+                ``variable_key`` is enabled.
             * rename_variables: dict, None
-                Mapping to rename variables. Default is None.
-            * extra_dims: str, or iterable of str, None
-                Metadata key or list of metadata keys to be used as additional dimensions on top of the
-                predefined dimensions. Only enabled when no ``fixed_dims`` is specified. Default is None.
+                Mapping to rename variables. Default is None. Only used  when
+                ``variable_key`` is enabled.
+            * mono_variable: bool, str, None
+                If True or str, the dataset will contain a single variable called "data" (or the value
+                of the ``mono_variable`` kwarg when it is a str). If False, the dataset will contain
+                one variable for each distinct value of ``variable_key`` metadata key. The default value
+                (None) expands to False unless the ``profile`` overwrites it.
+            * extra_dims:  str, or iterable of str, None
+                Define additional dimensions on top of the predefined dimensions. Only enabled when no
+                ``fixed_dims`` is specified. Default is None. It can be a single item or a list. Each
+                item is either a metadata key, or a dict/tuple defining mapping between the dimension
+                name and the metadata key. The whole option can be a dict. E.g.
+
+                .. code-block:: python
+
+                    # use key "expver" as a dimension
+                    extra_dims = "expver"
+                    # use keys "expver" and "steam" as a dimension
+                    extra_dims = ["expver", "stream"]
+                    # define dimensions "expver", mars_stream" and "mars_type" from
+                    # metadata keys "expver", "stream" and "type"
+                    extra_dims = [
+                        "expver",
+                        {"mars_stream": "stream"},
+                        ("mars_type", "type"),
+                    ]
+                    extra_dims = [
+                        {
+                            "expver": "expver",
+                            "mars_stream": "stream",
+                            "mars_type": "type",
+                        }
+                    ]
+
             * drop_dims:  str, or iterable of str, None
-                Metadata key or list of metadata keys to be ignored as dimensions. Default is None.
+                Single or multiple dimensions to be ignored. Default is None.
                 Default is None.
             * ensure_dims: str, or iterable of str, None
-                Metadata key or list of metadata keys that should be used as dimensions even
-                when ``squeeze=True``. Default is None.
+                Dimension or dimensions that should be kept even when ``squeeze=True`` and their size
+                is only 1. Default is None.
             * fixed_dims: str, or iterable of str, None
-                Metadata key or list of metadata keys in the order they should be used as dimensions. When
-                defined no other dimensions will be used. Might be incompatible with other settings.
-                Default is None.
+                Define all the dimensions to be generated. When used no other dimensions will be created.
+                Might be incompatible with other settings. Default is None. It can be a single item or a list.
+                Each item is either a metadata key, or a dict/tuple defining mapping between the dimension
+                name and the metadata key. The whole option can be a dict. E.g.:
+
+                .. code-block:: python
+
+                    # use key "step" as a dimension
+                    fixed_dims = "step"
+                    # use keys "step" and "levelist" as a dimension
+                    extra_dims = ["step", "levelist"]
+                    # define dimensions "step", level" and "level_type" from
+                    # metadata keys "step", "levelist" and "levtype"
+                    extra_dims = [
+                        "step",
+                        {"level": "levelist"},
+                        ("level_type", "levtype"),
+                    ]
+                    extra_dims = [
+                        {"step": "step", "level": "levelist", "level_type": "levtype"}
+                    ]
+
             * dim_roles: dict, None
                 Specify the "roles" used to form the predefined dimensions. The predefined dimensions are
                 automatically generated when no ``fixed_dims`` specified and comprise the following
@@ -138,7 +188,7 @@ class XarrayMixIn:
                 ``dim_roles`` is a mapping between the "roles" and the metadata keys representing the roles.
                 The possible roles are as follows:
 
-                - "ens": metadata key interpreted as ensemble forecast members
+                - "number": metadata key interpreted as ensemble forecast members
                 - "date": metadata key interpreted as date part of the "forecast_reference_time"
                 - "time": metadata key interpreted as time part of the "forecast_reference_time"
                 - "step": metadata key interpreted as forecast step
@@ -154,7 +204,7 @@ class XarrayMixIn:
                 .. code-block:: python
 
                     {
-                        "ens": "number",
+                        "number": "number",
                         "date": "dataDate",
                         "time": "dataTime",
                         "step": "step",
@@ -165,8 +215,13 @@ class XarrayMixIn:
                     }
 
                 ``dims_roles`` behaves differently to the other kwargs in the sense that
-                it does not override but update the default values. So e.g. to change only "ens" in
-                the defaults it is enough to specify: "dim_roles={"ens": "perturbationNumber"}.
+                it does not override but update the default values. So e.g. to change only "number" in
+                the defaults it is enough to specify: "dim_roles={"number": "perturbationNumber"}.
+            * dim_name_from_role_name: bool, None
+                If True, the dimension names are formed from the role names. Otherwise the
+                dimension names are formed from the metadata keys specified in ``dim_roles``.
+                Its default value (None) expands to True unless the ``profile`` overwrites it.
+                Only used when no `fixed_dims`` are specified. *New in version 0.15.0*.
             * rename_dims: dict, None
                 Mapping to rename dimensions. Default is None.
             * dims_as_attrs: str, or iterable of str, None
@@ -231,10 +286,12 @@ class XarrayMixIn:
                 - "fixed": Use the attributes defined in ``variable_attrs`` as variables
                   attributes and ``global_attrs`` as global attributes.
                 - "unique": Use all the attributes defined in ``attrs``, ``variable_attrs``
-                  and ``global_attrs``. When an attribute has unique a value for a dataset
+                  and ``global_attrs``.  When an attribute has unique value for a dataset
                   it will be a global attribute, otherwise it will be a variable attribute.
-                  However keys in ``variable_attrs`` are always used as variable attributes,
-                  while keys in ``global_attrs`` are always used as global attributes.
+                  However, this logic is only applied if a unique variable attribute can be
+                  a global attribute according to the CF conventions Appendix A. (e.g. "units" cannot
+                  be a global attribute). Additionally, keys in ``variable_attrs`` are always used as
+                  variable attributes, while keys in ``global_attrs`` are always used as global attributes.
             * attrs: str, number, callable, dict or list of these, None
                 Attribute or list of attributes. Only used when ``attrs_mode`` is ``unique``.
                 Its default value (None) expands to [] unless the ``profile`` overwrites it.
@@ -263,6 +320,8 @@ class XarrayMixIn:
                 (None) expands to True unless the ``profile`` overwrites it.
             * rename_attrs: dict, None
                 A dictionary of attribute to rename. Default is None.
+            * fill_metadata: dict, None
+                Define fill_metadata values to metadata keys. Default is None.
             * remapping: dict, None
                 Define new metadata keys for indexing. Default is None.
             * lazy_load: bool, None
@@ -286,8 +345,9 @@ class XarrayMixIn:
                 to False unless the ``profile`` overwrites it.
             * dtype: str, numpy.dtype or None
                 Typecode or data-type of the array data.
-            * array_module: module
-                The module to use for array operations. Default is numpy.
+            * array_backend: str, array namespace, ArrayBackend, None
+                The array backend/namespace to use for array operations. The default value (None) is
+                expanded to "numpy".
             * direct_backend: bool, None
                 If True, the backend is used directly bypassing :py:meth:`xarray.open_dataset`
                 and ignoring all non-backend related kwargs. If False, the data is read via
@@ -306,9 +366,14 @@ class XarrayMixIn:
 
         Returns
         -------
-        Xarray Dataset or list of Xarray Datasets
-
+        Xarray Dataset or tuple
+            When ``split_dims`` is unset a Dataset is returned. When ``engine="earthkit"``
+            and ``split_dims`` is set a tuple is returned. The first element of the tuple is the list
+            of Datasets and the second element is the list of corresponding dictionaries with
+            the spitting keys/values (one dictionary per Dataset).
 
+        Notes
+        -----
         The default values of ``xarray_open_dataset_kwargs`` or ``**kwargs`` passed
         to :py:func:`xarray.open_dataset` are as follows:
 
@@ -338,23 +403,12 @@ class XarrayMixIn:
         ... )
 
         """
-        if engine is None:
-            engine = "earthkit"
-            global _WARNED_ABOUT_EARTHKIT
-            if not _WARNED_ABOUT_EARTHKIT:
-                LOG.warning(
-                    (
-                        "From version 0.11.0 the default engine for to_xarray is 'earthkit'. "
-                        "Use engine=`cfgrib` to invoke the cfgrib engine."
-                    )
-                )
-                _WARNED_ABOUT_EARTHKIT = True
-
         engines = {"earthkit": self.to_xarray_earthkit, "cfgrib": self.to_xarray_cfgrib}
         if engine not in engines:
             raise ValueError(f"Unsupported engine: {engine}. Please use one of {list(engines.keys())}")
 
         user_xarray_open_dataset_kwargs = ensure_dict(xarray_open_dataset_kwargs)
+        user_xarray_open_dataset_kwargs = copy.deepcopy(user_xarray_open_dataset_kwargs)
         if user_xarray_open_dataset_kwargs and kwargs:
             raise ValueError("Cannot specify extra keyword arguments when xarray_open_dataset_kwargs is set.")
 
