@@ -44,9 +44,6 @@ class TextReader(Reader):
 
 
 def reader(source, path, *, magic=None, deeper_check=False, **kwargs):
-    if magic is None:  # Bypass check and force
-        return TextReader(source, path)
-
     if deeper_check:
         if is_text(path):
             return TextReader(source, path)
