@@ -11,7 +11,6 @@ import logging
 from abc import ABCMeta
 from abc import abstractmethod
 
-import numpy
 import xarray
 import xarray.core.indexing as indexing
 
@@ -32,7 +31,14 @@ NON_XR_OPEN_DS_KWARGS = ["split_dims", "direct_backend"]
 
 class VariableBuilder:
     def __init__(
-        self, name, var_dims, data_maker, tensor, remapping, local_attr_keys=None, fixed_local_attrs=None
+        self,
+        name,
+        var_dims,
+        data_maker,
+        tensor,
+        remapping,
+        local_attr_keys=None,
+        fixed_local_attrs=None,
     ):
         """
         Create a builder for a single variable in the dataset.
@@ -67,10 +73,13 @@ class VariableBuilder:
 
     def build(self, add_earthkit_attrs=True):
         if add_earthkit_attrs:
-            attrs = {
-                "message": self.tensor.source[0].metadata().override()._handle.get_buffer(),
-            }
-            self._attrs["_earthkit"] = attrs
+            if hasattr(self.tensor.source[0], "handle"):
+                md = self.tensor.source[0].metadata().override()
+                attrs = {
+                    "message": md._handle.get_buffer(),
+                    "bitsPerValue": md.get("bitsPerValue", 0),
+                }
+                self._attrs["_earthkit"] = attrs
 
         self._attrs.update(self.fixed_local_attrs)
         data = self.data_maker(self.tensor, self.var_dims, self.name)
@@ -133,7 +142,7 @@ class VariableBuilder:
                     fixed_attrs[a.name] = a.value()
                 elif callable(a):
                     res.update(a(_metadata()))
-                elif isinstance(a, str):
+                else:
                     res.update(a.get(_metadata()))
 
             res = {k: v for k, v in res.items() if v is not None}
@@ -163,19 +172,18 @@ class VariableBuilder:
 
 
 class TensorBackendArray(xarray.backends.common.BackendArray):
-    def __init__(self, tensor, dims, shape, xp, dtype, variable):
+    def __init__(self, tensor, dims, shape, xp, dtype, var_name):
         super().__init__()
         self.tensor = tensor
         self.dims = dims
         self.shape = shape
+        self._var_name = var_name
+        self.dtype = dtype
+        self.xp = xp
 
-        # xp and dtype must be set for xarray
-        self.xp = xp if xp is not None else numpy
-        if dtype is None:
-            dtype = numpy.dtype("float64")
-        self.dtype = xp.dtype(dtype)
+        from dask.utils import SerializableLock
 
-        self._var = variable
+        self.lock = SerializableLock()
 
     @property
     def nbytes(self):
@@ -203,39 +211,35 @@ class TensorBackendArray(xarray.backends.common.BackendArray):
         # patched in local copy for now, but could construct this ourself
 
     def _raw_indexing_method(self, key: tuple):
-        # must be threadsafe
-        # print("_var", self._var)
-        # print(f"dims: {self.dims} key: {key} shape: {self.shape}")
-        # isels = dict(zip(self.dims, key))
-        # r = self.ekds.isel(**isels)
-        # print(f"t-coords={self.tensor.user_coords}")
-        r = self.tensor[key]
-        # print(r.source.ls())
-        # print(f"r-shape: {r.user_shape}")
+        with self.lock:
+            # LOG.debug(f"TensorBackendArray._raw_indexing_method var={self._var_name}")
+            # LOG.debug(f"   dims={self.dims} key={key} shape={self.shape}")
+            # LOG.debug(f"   tensor.user_coords={self.tensor.user_coords}")
 
-        field_index = r.field_indexes(key)
-        # print(f"field.index={field_index} coords={r.user_coords}")
-        # result = r.to_numpy(index=field_index).squeeze()
-        result = r.to_numpy(index=field_index, dtype=self.dtype)
+            r = self.tensor[key]
+            # LOG.debug(f"   cubelet user_shape={r.user_shape}")
 
-        # ensure axes are squeezed when needed
-        singles = [i for i in list(range(len(r.user_shape))) if isinstance(key[i], int)]
-        if singles:
-            result = result.squeeze(axis=tuple(singles))
+            # LOG.debug(f"   {r.user_shape=}")
+            field_index = r.field_indexes(key)
+            if self.tensor.is_full_field(field_index):
+                field_index = None
 
-        # print("result", result.shape)
-        # result = self.ekds.isel(**isels).to_numpy()
+            # LOG.debug(f"   {field_index=}")
 
-        # print("result", result.shape)
-        # print(f"Loaded {self.xp.__name__} with shape: {result.shape}")
+            try:
+                result = r.to_array(index=field_index, array_backend=self.xp, dtype=self.dtype)
+            except Exception as e:
+                LOG.exception("Error in to_array:", e)
+                raise
 
-        # Loading as numpy but then converting. This needs to be changed upstream (eccodes)
-        # to load directly into cupy.
-        # Maybe some incompatibilities when trying to copy from FFI to cupy directly
-        if self.xp and self.xp != numpy:
-            result = self.xp.asarray(result)
+            # LOG.debug(f"   {result.shape=}"
 
-        return result
+            # ensure axes are squeezed when needed
+            singles = [i for i in list(range(len(r.user_shape))) if isinstance(key[i], int)]
+            if singles:
+                result = result.squeeze(axis=tuple(singles))
+
+            return result
 
 
 class BackendDataBuilder(metaclass=ABCMeta):
@@ -245,8 +249,21 @@ class BackendDataBuilder(metaclass=ABCMeta):
         self.dims = dims
 
         self.flatten_values = profile.flatten_values
-        self.dtype = profile.dtype
-        self.array_module = profile.array_module
+
+        # Array backend/namespace
+        array_backend = profile.array_backend
+        if array_backend is None:
+            array_backend = "numpy"
+
+        from earthkit.utils.array import get_backend
+
+        self.array_backend = get_backend(array_backend)
+        assert self.array_backend is not None, f"Unsupported array_backend : {array_backend}"
+
+        dtype = profile.dtype
+        if dtype is None:
+            dtype = "float64"
+        self.dtype = self.array_backend.make_dtype(dtype)
 
         # Note: these coords inside the tensor are called user_coords and
         # the corresponding dims are called user_dims
@@ -289,7 +306,7 @@ class BackendDataBuilder(metaclass=ABCMeta):
         ):
             from .coord import Coord
 
-            _dims, _vals = tensor.make_valid_datetime()
+            _dims, _vals = tensor.make_valid_datetime(self.dims)
             if _dims is not None and _vals is not None:
                 self.tensor_coords["valid_time"] = Coord.make("valid_time", _vals, dims=_dims)
 
@@ -303,15 +320,20 @@ class BackendDataBuilder(metaclass=ABCMeta):
 
         # build variables
         xr_vars = {
-            self.profile.rename_variable(k): v.build(add_earthkit_attrs=self.profile.add_earthkit_attrs)
+            self.profile.variable.rename(k): v.build(add_earthkit_attrs=self.profile.add_earthkit_attrs)
             for k, v in var_builders.items()
         }
 
         # build dataset
         dataset = xarray.Dataset(xr_vars, coords=xr_coords, attrs=xr_attrs)
 
-        if self.profile.rename_dims_map():
-            dataset = dataset.rename(self.profile.rename_dims_map())
+        dataset = self.profile.rename_dataset_dims(dataset)
+
+        # dim_map = self.profile.rename_dims_map()
+        # if dim_map:
+        #     d = {k: v for k, v in dim_map.items() if k in dataset.dims}
+        #     if d:
+        #         dataset = dataset.rename(d)
 
         if "source" not in dataset.encoding:
             dataset.encoding["source"] = None
@@ -407,7 +429,7 @@ class BackendDataBuilder(metaclass=ABCMeta):
                 elif num > 1 or not self.profile.dims.squeeze or d.name in self.profile.dims.ensure_dims:
                     tensor_dims.append(d)
                     tensor_coords[d.key] = vals[d.key]
-                    if d.key in component_vals:
+                    if component_vals and d.key in component_vals:
                         tensor_coords_component[d.key] = component_vals[d.key]
 
                     # check if the dims/coords are consistent with the tensors of
@@ -435,20 +457,29 @@ class TensorBackendDataBuilder(BackendDataBuilder):
         """Generate a builder for each variable"""
         builders = {}
 
-        # we assume each variable forms a full cube
-        for name in self.profile.variables:
-            ds_var = self.ds.sel(**{self.profile.variable_key: name})
-            builders[name] = self.pre_build_variable(ds_var, self.dims, name)
+        if self.profile.variable.is_mono:
+            name = self.profile.variable.name
+            builders[name] = self.pre_build_variable(self.ds, self.dims, name)
+        else:
+            # we assume each variable forms a full cube
+            key = self.profile.variable.key
+            for name in self.profile.variable.variables:
+                ds_var = self.ds.sel(**{key: name})
+                builders[name] = self.pre_build_variable(ds_var, self.dims, name)
 
         return builders
 
     def build_values(self, tensor, var_dims, name):
         """Generate the data object stored in the xarray variable"""
+        # There is no need for the extra structures in the wrapped source in the
+        # tensor any longer. It is replaced by the original unwrapped fieldlist.
+        tensor.source = tensor.source.unwrap()
+
         backend_array = TensorBackendArray(
             tensor,
             var_dims,
             tensor.full_shape,
-            self.array_module,
+            self.array_backend.namespace,
             self.dtype,
             name,
         )
@@ -468,12 +499,17 @@ class MemoryBackendDataBuilder(BackendDataBuilder):
     def pre_build_variables(self):
         """Generate a builder for each variable"""
         builders = {}
-        groups = self.ds.group(self.profile.variable_key, self.profile.variables)
 
-        # we assume each variable forms a full cube
-        for name in groups:
-            ds_var = groups[name]
-            builders[name] = self.pre_build_variable(ds_var, self.dims, name)
+        if self.profile.variable.is_mono:
+            name = self.profile.variable.name
+            builders[name] = self.pre_build_variable(self.ds, self.dims, name)
+        else:
+            groups = self.ds.group(self.profile.variable.key, self.profile.variable.variables)
+
+            # we assume each variable forms a full cube
+            for name in groups:
+                ds_var = groups[name]
+                builders[name] = self.pre_build_variable(ds_var, self.dims, name)
 
         return builders
 
@@ -487,7 +523,7 @@ class MemoryBackendDataBuilder(BackendDataBuilder):
             for f in tensor.source:
                 f.keep = False
 
-        return tensor.to_numpy(dtype=self.dtype)
+        return tensor.to_array(dtype=self.dtype, array_backend=self.array_backend)
 
 
 class DatasetBuilder:
@@ -532,7 +568,7 @@ class DatasetBuilder:
 
         # LOG.debug(f"{remapping=}")
         # LOG.debug(f"{profile.remapping=}")
-        # LOG.debug(f"{profile.index_keys=}")
+        LOG.debug(f"{profile.index_keys=}")
 
         # create a new fieldlist for optimised access to unique values
         ds_xr = XArrayInputFieldList(
@@ -556,15 +592,19 @@ class DatasetBuilder:
     def grid(self, ds):
         grids = ds.index("md5GridSection")
 
-        if len(grids) != 1:
-            raise ValueError(f"Expected one grid, got {len(grids)}")
-        grid = grids[0]
+        if not grids:
+            grid = "_custom_" + str(id(ds))
+        else:
+            # if len(grids) != 1:
+            #     raise ValueError(f"Expected one grid, got {len(grids)}")
+            grid = grids[0]
+
         key = (grid, self.profile.flatten_values)
 
         if key not in self.grids:
             from .grid import TensorGrid
 
-            self.grids = {key: TensorGrid(ds[0], self.profile.flatten_values)}
+            self.grids[key] = TensorGrid(ds[0], self.profile.flatten_values)
         return self.grids[key]
 
 
@@ -620,7 +660,8 @@ class SplitDatasetBuilder(DatasetBuilder):
         # LOG.debug(f"split_dims={self.split_dims}")
         ds_xr = XArrayInputFieldList(self.ds, keys=self.profile.index_keys, remapping=remapping)
 
-        vals, _ = ds_xr.unique_values(*keys)
+        vals, _ = ds_xr.unique_values(keys)
+        LOG.debug(f"{keys=}, {vals=}")
 
         return ds_xr, vals
 
@@ -629,10 +670,12 @@ class SplitDatasetBuilder(DatasetBuilder):
 
         splitter = Splitter.make(self.split_dims)
         datasets = []
+        split_coords_list = []
         for ds, profile, split_coords in splitter.split(self):
             dims = profile.dims.to_list()
+            split_coords_list.append(dict(split_coords))
             LOG.debug(f"splitting {dims=} type of s_ds={type(ds)} {split_coords=}")
-            split_coords.pop(profile.variable_key, None)
+            split_coords.pop(profile.variable.key, None)
             builder = self.builder(ds, profile, dims, grid=self.grid(ds), fixed_local_attrs=split_coords)
 
             ds._ek_builder = builder
@@ -643,7 +686,7 @@ class SplitDatasetBuilder(DatasetBuilder):
                 datasets.append(xarray.open_dataset(ds, **self.xr_open_dataset_kwargs))
                 ds._ek_builder = None
 
-        return datasets[0] if len(datasets) == 1 else datasets
+        return datasets, split_coords_list
 
 
 def from_earthkit(ds, backend_kwargs=None, other_kwargs=None):
