@@ -15,8 +15,8 @@ import sys
 
 import pytest
 
+from earthkit.data import config
 from earthkit.data import from_source
-from earthkit.data import settings
 from earthkit.data.testing import earthkit_examples_file
 from earthkit.data.utils.diag import field_cache_diag
 
@@ -65,7 +65,7 @@ def _check_diag(diag, ref):
 @pytest.mark.parametrize("serialise", [True, False])
 def test_grib_cache_basic_file_patched(handle_cache_size, serialise, patch_metadata_cache):
 
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "persistent",
             "grib-handle-policy": "cache",
@@ -149,7 +149,7 @@ def test_grib_cache_basic_file_non_patched():
     """This test is the same as test_grib_cache_basic but without the patch_metadata_cache fixture.
     So metadata cache hits and misses are not counted."""
 
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "persistent",
             "grib-handle-policy": "cache",
@@ -221,7 +221,7 @@ def test_grib_cache_basic_file_non_patched():
 @pytest.mark.parametrize("fl_type", ["file", "array", "memory"])
 def test_grib_cache_basic_metadata_patched(serialise, fl_type, patch_metadata_cache):
 
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "persistent",
             "grib-handle-policy": "cache",
@@ -287,7 +287,7 @@ def test_grib_cache_basic_metadata_patched(serialise, fl_type, patch_metadata_ca
 
 
 def test_grib_cache_options_1(patch_metadata_cache):
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "persistent",
             "grib-handle-policy": "temporary",
@@ -367,7 +367,7 @@ def test_grib_cache_options_1(patch_metadata_cache):
 
 
 def test_grib_cache_options_2(patch_metadata_cache):
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "persistent",
             "grib-handle-policy": "persistent",
@@ -449,7 +449,7 @@ def test_grib_cache_options_2(patch_metadata_cache):
 
 
 def test_grib_cache_options_3(patch_metadata_cache):
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "persistent",
             "grib-handle-policy": "cache",
@@ -529,7 +529,7 @@ def test_grib_cache_options_3(patch_metadata_cache):
 
 
 def test_grib_cache_options_4(patch_metadata_cache):
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "temporary",
             "grib-handle-policy": "temporary",
@@ -626,7 +626,7 @@ def test_grib_cache_options_4(patch_metadata_cache):
 
 
 def test_grib_cache_options_5(patch_metadata_cache):
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "temporary",
             "grib-handle-policy": "persistent",
@@ -725,7 +725,7 @@ def test_grib_cache_options_5(patch_metadata_cache):
 
 
 def test_grib_cache_options_6(patch_metadata_cache):
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "temporary",
             "grib-handle-policy": "cache",
@@ -858,7 +858,7 @@ def test_grib_cache_file_use_kwargs_2():
 
 @pytest.mark.parametrize("fl_type", ["file", "array", "memory"])
 def test_grib_cache_metadata_use_kwargs_1(fl_type, patch_metadata_cache):
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "persistent",
             "grib-handle-policy": "cache",
@@ -900,7 +900,7 @@ def test_grib_cache_metadata_use_kwargs_1(fl_type, patch_metadata_cache):
 
 @pytest.mark.parametrize("fl_type", ["file", "array", "memory"])
 def test_grib_cache_metadata_use_kwargs_2(fl_type, patch_metadata_cache):
-    with settings.temporary(
+    with config.temporary(
         {
             "grib-field-policy": "persistent",
             "grib-handle-policy": "cache",
