@@ -9,9 +9,13 @@
 # nor does it submit to any jurisdiction.
 #
 
+import pytest
+
 from earthkit.data import from_source
 from earthkit.data.core.temporary import temp_file
+from earthkit.data.testing import WRITE_TO_FILE_METHODS
 from earthkit.data.testing import earthkit_test_data_file
+from earthkit.data.testing import write_to_file
 
 
 def test_grib_from_memory():
@@ -24,12 +28,13 @@ def test_grib_from_memory():
         assert fs[0].metadata("shortName") == "2t"
 
 
-def test_grib_save_when_loaded_from_memory():
+@pytest.mark.parametrize("write_method", WRITE_TO_FILE_METHODS)
+def test_grib_save_when_loaded_from_memory(write_method):
     with open(earthkit_test_data_file("test_single.grib"), "rb") as f:
         data = f.read()
         fs = from_source("memory", data)
         with temp_file() as tmp:
-            fs.save(tmp)
+            write_to_file(write_method, tmp, fs)
             fs_saved = from_source("file", tmp)
             assert len(fs) == len(fs_saved)
 
