@@ -22,17 +22,12 @@ from earthkit.data.testing import NO_HDA
 def test_wekeo_download(prompt):
     s = from_source(
         "wekeo",
-        "EO:CLMS:DAT:CGLS_GLOBAL_NDVI300_V1_333M",
+        "EO:CLMS:DAT:CLMS_GLOBAL_BA_300M_V3_MONTHLY_NETCDF",
         prompt=prompt,
         request={
-            "datasetId": "EO:CLMS:DAT:CGLS_GLOBAL_NDVI300_V1_333M",
-            "dateRangeSelectValues": [
-                {
-                    "name": "dtrange",
-                    "start": "2014-01-01T00:00:00.000Z",
-                    "end": "2014-01-01T23:59:59.999Z",
-                }
-            ],
+            "dataset_id": "EO:CLMS:DAT:CLMS_GLOBAL_BA_300M_V3_MONTHLY_NETCDF",
+            "startdate": "2019-01-01T00:00:00.000Z",
+            "enddate": "2019-01-01T23:59:59.999Z",
         },
     )
     assert len(s) == 1
